@@ -10,7 +10,7 @@ use std::io;
 use std::io::prelude::*;
 use std::time::SystemTime;
 
-use crate::op_analysis::MemoryAccess;
+use crate::op_analysis::{InstructionPointerUpdate, InstructionProperties, MemoryAccessList};
 use minidump::system_info::PointerWidth;
 use minidump::*;
 use minidump_common::utils::basename;
@@ -217,12 +217,18 @@ pub struct ExceptionInfo {
     pub adjusted_address: Option<AdjustedAddress>,
     /// A string representing the crashing instruction (if available)
     pub instruction_str: Option<String>,
+    /// A list of booleans representing properties of crashing instruction (if availaable)
+    pub instruction_properties: Option<InstructionProperties>,
     /// A list of memory accesses performed by crashing instruction (if available)
-    pub memory_accesses: Option<Vec<MemoryAccess>>,
+    pub memory_access_list: Option<MemoryAccessList>,
+    /// Whether the instruction pointer is updated by crashing instruction (if available)
+    pub instruction_pointer_update: Option<InstructionPointerUpdate>,
     /// Possible valid addresses which are one flipped bit away from the crashing address or adjusted address.
     ///
     /// The original address was possibly the result of faulty hardware, alpha particles, etc.
     pub possible_bit_flips: Vec<PossibleBitFlip>,
+    /// Whether the crash reason/address is inconsistent with crashing instruction and memory info
+    pub inconsistencies: Vec<CrashInconsistency>,
 }
 
 /// Info about a memory address that was adjusted from its reported value
@@ -416,6 +422,38 @@ impl PossibleBitFlip {
     }
 }
 
+#[derive(serde::Serialize, Debug, Clone)]
+#[serde(rename_all = "snake_case")]
+pub enum CrashInconsistency {
+    IntDivByZeroNotPossible,
+    PrivInstructionCrashWithoutPrivInstruction,
+    NonCanonicalAddressFalselyReported,
+    AccessViolationWhenAccessAllowed,
+    CrashingAccessNotFoundInMemoryAccesses,
+}
+
+impl std::fmt::Display for CrashInconsistency {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            CrashInconsistency::IntDivByZeroNotPossible => {
+                f.write_str("Crash reason is an integer division by zero but the crashing instruction is not a division")
+            }
+            CrashInconsistency::PrivInstructionCrashWithoutPrivInstruction => {
+                f.write_str("Crash reason is a privileged instruction but crashing instruction is not a privileged one")
+            }
+            CrashInconsistency::NonCanonicalAddressFalselyReported => {
+                f.write_str("Crash address is reported as a non-canonical x86-64 address but the actual address is a canonical one")
+            }
+            CrashInconsistency::AccessViolationWhenAccessAllowed => {
+                f.write_str("Crash reason is access violation exception but access is allowed")
+            }
+            CrashInconsistency::CrashingAccessNotFoundInMemoryAccesses => f.write_str(
+                "Crash address not found among the memory accesses of the crashing instruction",
+            ),
+        }
+    }
+}
+
 /// The state of a process as recorded by a `Minidump`.
 #[derive(Debug, Clone)]
 pub struct ProcessState {
@@ -464,6 +502,7 @@ pub struct ProcessState {
     pub unknown_streams: Vec<MinidumpUnknownStream>,
     pub unimplemented_streams: Vec<MinidumpUnimplementedStream>,
     pub symbol_stats: HashMap<String, SymbolStats>,
+    pub linux_memory_map_count: Option<usize>,
 }
 
 fn json_registers(ctx: &MinidumpContext) -> serde_json::Value {
@@ -564,25 +603,47 @@ impl ProcessState {
                 writeln!(f, "Crashing instruction: `{crashing_instruction_str}`")?;
             }
 
-            if let Some(ref memory_accesses) = crash_info.memory_accesses {
-                if !memory_accesses.is_empty() {
+            if let Some(ref access_list) = crash_info.memory_access_list {
+                if !access_list.is_empty() {
                     writeln!(f, "Memory accessed by instruction:")?;
-                    for (idx, access) in memory_accesses.iter().enumerate() {
-                        writeln!(f, "  {idx}. Address: {}", Address(access.address))?;
+                    for (idx, access) in access_list.iter().enumerate() {
+                        writeln!(
+                            f,
+                            "  {idx}. Address: {}",
+                            Address(access.address_info.address)
+                        )?;
                         if let Some(size) = access.size {
                             writeln!(f, "     Size: {size}")?;
                         } else {
                             writeln!(f, "     Size: Unknown")?;
                         }
-                        if access.is_likely_guard_page {
+                        if access.address_info.is_likely_guard_page {
                             writeln!(f, "     This address falls in a likely guard page.")?;
                         }
+                        if access.access_type.is_read_or_write() {
+                            writeln!(f, "     Access type: {}", access.access_type)?;
+                        }
                     }
                 } else {
                     writeln!(f, "No memory accessed by instruction")?;
                 }
             }
 
+            if let Some(ref rip_update) = crash_info.instruction_pointer_update {
+                match rip_update {
+                    InstructionPointerUpdate::Update { address_info } => {
+                        writeln!(f, "Instruction pointer update done by instruction:")?;
+                        writeln!(f, "  Address: {}", Address(address_info.address))?;
+                        if address_info.is_likely_guard_page {
+                            writeln!(f, "     This address falls in a likely guard page.")?;
+                        }
+                    }
+                    InstructionPointerUpdate::NoUpdate => {
+                        writeln!(f, "No instruction pointer update by instruction")?;
+                    }
+                }
+            }
+
             if !crash_info.possible_bit_flips.is_empty() {
                 writeln!(f, "Crashing address may be the result of a flipped bit:")?;
                 let mut bit_flips_with_confidence = crash_info
@@ -609,6 +670,12 @@ impl ProcessState {
                     )?;
                 }
             }
+            if !crash_info.inconsistencies.is_empty() {
+                writeln!(f, "Crash is inconsistent:")?;
+                for inconsistency in &crash_info.inconsistencies {
+                    writeln!(f, "  {}", inconsistency)?;
+                }
+            }
         } else {
             writeln!(f, "No crash")?;
         }
@@ -664,18 +731,24 @@ impl ProcessState {
         }
         writeln!(f)?;
 
+        if let Some(linux_memory_map_count) = self.linux_memory_map_count {
+            writeln!(f, "Linux memory map count: {}", linux_memory_map_count)?;
+            writeln!(f)?;
+        }
+
         if let Some(requesting_thread) = self.requesting_thread {
             let stack = &self.threads[requesting_thread];
             writeln!(
                 f,
-                "Thread {} {} ({})",
+                "Thread {} {} ({}) - tid: {}",
                 requesting_thread,
                 stack.thread_name.as_deref().unwrap_or(""),
                 if self.crashed() {
                     "crashed"
                 } else {
                     "requested dump, did not crash"
-                }
+                },
+                stack.thread_id
             )?;
             stack.print(f)?;
             writeln!(f)?;
@@ -696,9 +769,10 @@ impl ProcessState {
             }
             writeln!(
                 f,
-                "Thread {} {}",
+                "Thread {} {} - tid: {}",
                 i,
-                stack.thread_name.as_deref().unwrap_or("")
+                stack.thread_name.as_deref().unwrap_or(""),
+                stack.thread_id
             )?;
             stack.print(f)?;
         }
@@ -832,23 +906,47 @@ Unknown streams encountered:
                 }),
                 "instruction": self.exception_info.as_ref().map(|info| info.instruction_str.as_ref()),
                 "memory_accesses": self.exception_info.as_ref().and_then(|info| {
-                    info.memory_accesses.as_ref().map(|accesses| {
-                        accesses.iter().map(|access| {
+                    info.memory_access_list.as_ref().map(|access_list| {
+                        access_list.iter().map(|access| {
                             let mut map = json!({
-                                "address": json_hex(access.address),
+                                "address": json_hex(access.address_info.address),
                                 "size": access.size,
                             });
                             // Only add the `is_likely_guard_page` field when it is affirmative.
-                            if access.is_likely_guard_page {
+                            if access.address_info.is_likely_guard_page {
                                 map["is_likely_guard_page"] = true.into();
                             }
+                            if access.access_type.is_read_or_write() {
+                                map["access_type"] = access.access_type.to_string().to_lowercase().into();
+                            }
                             map
                         }).collect::<Vec<_>>()
                     })
                 }),
+                "instruction_pointer_update": self.exception_info.as_ref().and_then(|info| {
+                    info.instruction_pointer_update.as_ref().map(|update| {
+                        match update {
+                            InstructionPointerUpdate::Update { address_info } => {
+                                let mut map = json!({
+                                    "address": json_hex(address_info.address),
+                                });
+                                if address_info.is_likely_guard_page {
+                                    map["is_likely_guard_page"] = true.into();
+                                }
+                                map
+                            }
+                            InstructionPointerUpdate::NoUpdate => {
+                                json!(null)
+                            }
+                        }
+                    })
+                }),
                 "possible_bit_flips": self.exception_info.as_ref().and_then(|info| {
                     (!info.possible_bit_flips.is_empty()).then_some(&info.possible_bit_flips)
                 }),
+                "crash_inconsistencies": self.exception_info.as_ref().map(|info| {
+                    &info.inconsistencies
+                }),
                 // thread index | null
                 "crashing_thread": self.requesting_thread,
                 "assertion": self.assertion,
@@ -888,6 +986,9 @@ Unknown streams encountered:
             // optional
             "mac_boot_args": self.mac_boot_args.as_ref().map(|info| info.bootargs.as_ref()),
 
+            // optional
+            "linux_memory_map_count": self.linux_memory_map_count,
+
             // the first module is always the main one
             "main_module": 0,
             // [UNSTABLE:evil_json]
@@ -949,6 +1050,7 @@ Unknown streams encountered:
                 "last_error_value": thread.last_error_value.map(|error| error.to_string()),
                 // optional
                 "thread_name": thread.thread_name,
+                "thread_id" : thread.thread_id,
                 "frames": thread.frames.iter().enumerate().map(|(idx, frame)| json!({
                     "frame": idx,
                     // optional
