@@ -5,10 +5,10 @@ use std::fs::File;
 use std::io::Write;
 use std::ops::Deref;
 use std::panic;
+use std::path::PathBuf;
 use std::str::FromStr;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::time::Duration;
-use std::{boxed::Box, path::PathBuf};
 
 use indicatif::{MultiProgress, ProgressBar, ProgressStyle};
 use minidump::*;
@@ -425,8 +425,19 @@ async fn main_result() -> std::io::Result<()> {
 
             let mut provider = MultiSymbolProvider::new();
 
+            let modules = dump.get_stream::<MinidumpModuleList>().unwrap_or_default();
+
             if cli.use_local_debuginfo {
-                provider.add(Box::<DebugInfoSymbolProvider>::default());
+                let system_info = match dump.get_stream::<MinidumpSystemInfo>() {
+                    Err(e) => {
+                        error!("Error getting system info stream from dump (required for local debug info): {}", e);
+                        std::process::exit(1);
+                    }
+                    Ok(s) => s,
+                };
+                provider.add(Box::new(
+                    DebugInfoSymbolProvider::new(&system_info, &modules).await,
+                ));
             }
 
             if !cli.symbols_url.is_empty() {
